ProxyController = function () {
    // Global status that states if the custom proxy is set.
    // Listen on Browser Action clicks.
    chrome.browserAction.onClicked.addListener(this.onBrowserActionClicked.bind(this));
};
ProxyController.DOMAINS = ["api.stealthy.co", "api.stealthyapp.com", "api-stealthy.eu-central-1.elasticbeanstalk.com", "52.59.22.41", "52.58.254.187"];
ProxyController.domainsInUse = ProxyController.DOMAINS.slice();
ProxyController.currentDomain;

ProxyController.context = '';
ProxyController.ONLINE_ICON = '/img/online.png';
ProxyController.OFFLINE_ICON = '/img/offline.png';
ProxyController.WAIT_ICON = '/img/wait.png';
ProxyController.ERROR_MSG = "There is a problem communicating with the server, please try again later";
ProxyController.ON_MSG = "Stealthy is enabled, click to disable";
ProxyController.OFF_MSG = "Stealthy is disabled, click to enable";
ProxyController.WAIT_MSG = "Please wait";
ProxyController.errores = 0;
ProxyController.intentos = 0;
ProxyController.proxyArray = [];
ProxyController.wait = false;
ProxyController.timeProxyListRequest = null;
// Global status that states if the custom proxy is set.
ProxyController.proxyIsOn = false;
ProxyController.currentProxyID = -1;
ProxyController.allProxyInfos = [];

function handleProxyRequest(requestInfo) {
    //console.log(`Proxying: ${requestInfo.url}`);
    const url = new URL(requestInfo.url);
    // Determine whether the domain in the web address is on the blocked hosts list
    if ((ProxyController.proxyIsOn) && (ProxyController.DOMAINS.indexOf(url.hostname) == -1)) {
        return ProxyController.allProxyInfos;
    } else {
        return {type: "direct"};
    }

}

browser.proxy.onRequest.addListener(handleProxyRequest, {urls: ["<all_urls>"]});

browser.proxy.onError.addListener(error => {
    console.error(`Proxy error: ${error.message}`);
});

browser.contextMenus.create({
    id: "stealthy-options",
    title: "Options",
    contexts: ["browser_action"]
}, onMenuCreated);

function onMenuCreated() {
    if (browser.runtime.lastError) {
        console.log(`Error: ${browser.runtime.lastError}`);
    }
}

/**
 * To know the status when the custom proxy server is active or online.
 *
 * @returns {boolean} true is custom proxy is set and active.
 */
ProxyController.prototype.isProxyActive = function () {
    // return this.proxyStatus;
    return ProxyController.proxyIsOn;
};

browser.contextMenus.onClicked.addListener((info, tab) => {
    if (info.menuItemId != "stealthy-options") {
        return;
    }
    browser.runtime.openOptionsPage();
});


function handleInstalled(details) {
    if (details.reason == "install") {
        browser.tabs.create({
            url: "https://www.stealthy.co/Firefox-Installed/"
        });
        return;
    }

    if (details.reason == "update") {
        let manifest = browser.runtime.getManifest();
        let currVersion = manifest.version;
        let currentVersionParts = currVersion.split(".");
        let previousVersionParts = details.previousVersion.split(".");
        console.log("*****currVersion: " + currVersion);
        console.log("*****details.previousVersion: " + details.previousVersion);
        //length >= 2 should always be true
        if (currentVersionParts.length >= 2 && previousVersionParts.length >= 2) {
            if (currentVersionParts[0] != previousVersionParts[0] || currentVersionParts[1] != previousVersionParts[1]) {
                console.log("*****Mayor update, open URL");
                browser.tabs.create({
                    url: "https://www.stealthy.co/Firefox-Updated/"
                });
            } else {
                console.log("*****Minor update, ignore");
            }
        }
        return;
    }
    console.log("*****nor update or install, do nothing");
}

browser.runtime.onInstalled.addListener(handleInstalled);

/**
 * Browser action button on the Chrome toolbar that has been clicked. Toggle
 * behaviour.
 *
 * @parm {Object<Tab>} tab Gives the state of the tab that was clicked on.
 */
ProxyController.prototype.onBrowserActionClicked = function (tab) {
    if (!ProxyController.wait) {
        this.setProxyEnabled(!ProxyController.proxyIsOn);
    }
};

/**
 * Initialize the proxy.
 */
ProxyController.prototype.init = function () {
    // Check if the autostart setting is enabled, if it is, automatically start
    // our custom proxy server.
    this.setProxyEnabled(settings.autostart);
};
/**
 * Sets the current proxy server.
 *
 * @param {boolean}
 *            status_ True to turn it on, otherwise use the auto_detect option
 *            to bring it back to normal.
 */
ProxyController.prototype.setProxyEnabled = function (status_) {
    ProxyController.proxyIsOn = status_;
    if (status_) {
        if (0 == ProxyController.intentos) {
            ProxyController.getProxiesFromServer();
            return;
        }

        //10 minutes
        let proxyListExpirationDate = new Date();
        proxyListExpirationDate.setTime(ProxyController.timeProxyListRequest.getTime() + (10 * 60000));

        if (new Date() > proxyListExpirationDate) {
            if (settings.stealthy_type < 3) {
                //is NO PRO
                ProxyController.getProxiesFromServer();
                return;
            }
        }
        ProxyController.prototype.setStealthyProxy(true);
    } else {
        ProxyController.prototype.setStealthyProxy(false);
    }
};
ProxyController.prototype.setStealthyProxy = function (status_) {
    chrome.browserAction.setTitle({
        title: status_ ? ProxyController.ON_MSG : ProxyController.OFF_MSG
    });
    chrome.browserAction.setIcon({
        path: status_ ? ProxyController.ONLINE_ICON : ProxyController.OFFLINE_ICON
    });
};

ProxyController.getProxiesFromServer = function () {
    if (ProxyController.domainsInUse.length > 0) {
        ProxyController.currentDomain = ProxyController.domainsInUse[0];
    } else {
        console.log("ProxyController.domainsInUse empty, problems");
        ProxyController.prototype.setProxyEnabled(false);
        ProxyController.wait = false;

        //leave array full for next time (not automatic or recursion)
        ProxyController.domainsInUse = ProxyController.DOMAINS.slice();
        alert(ProxyController.ERROR_MSG);
        return;//important, quit here, do not continue.
    }

    ProxyController.wait = true;

    chrome.browserAction.setTitle({
        title: ProxyController.WAIT_MSG
    });
    chrome.browserAction.setIcon({
        path: ProxyController.WAIT_ICON
    });

    let req = new XMLHttpRequest();
    let blockads = "";
    if (settings.blockads) {
        blockads = "&blockads=true"
    }
    let parameters = '';
    if (4 == settings.stealthy_type) { //proUK
        parameters = 'type=pro&countryCode=UK' + blockads;
    } else if (3 == settings.stealthy_type) { //pro
        parameters = 'type=pro' + blockads;
    } else if (2 == settings.stealthy_type) {
        parameters = 'countryCode=' + settings.countryID;
    } else if (5 == settings.stealthy_type) {
        parameters = 'type=pro&countryCode=DE' + blockads;
    } else if (6 == settings.stealthy_type) {
        parameters = 'type=pro&countryCode=JP' + blockads;
    }

    //console.log("DOMAIN_IN_USE: " + ProxyController.currentDomain);
    //var tocall = 'http://' + ProxyController.currentDomain + ProxyController.context + '/GetProxy' + parameters;
    let tocall = 'http://' + ProxyController.currentDomain + ProxyController.context + '/GetProxy'
    //req.open("GET", tocall, true);
    req.open("POST", tocall, true);
    req.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
    req.onload = function () {
        if ((req.readyState == 4) && (req.status == 200)) {
            //2018: PRO servers first, shuffle the rest
            let allProxiesFromServer = JSON.parse(req.responseText);
            let ownFreeServers = [];
            let externalServers = [];
            allProxiesFromServer.forEach(function (item) {
                //IDs till 100 reserved for own free proxies
                if (item.id < 100) {
                    ownFreeServers.push(item);
                } else {
                    externalServers.push(item);
                }
            });
            ProxyController.proxyArray = shuffle(ownFreeServers).concat(shuffle(externalServers))

            ProxyController.allProxyInfos=[];
            for (let i = 0; i < ProxyController.proxyArray.length; i++) {
                let currentProxy = ProxyController.proxyArray[i];
                let proxyHost = currentProxy.host;
                let proxyPort = currentProxy.port;
                ProxyController.allProxyInfos.push({type: "http", host: proxyHost, port: proxyPort})
            }

            //ProxyController.proxyArray = JSON.parse(req.responseText);
            ProxyController.prototype.setStealthyProxy(true);
            ProxyController.timeProxyListRequest = new Date();
        } else {
            // getting 403 (Forbidden) on some public wifi
            console.log("req.status: " + req.status);
            ProxyController.fail();
        }
        ProxyController.wait = false;
    };
    req.onerror = function () {
        console.log("req.onerror");
        ProxyController.fail();
    };
    parameters += "&secret=NotSoSecret";
    try {
        req.send(parameters);
    } catch (e) {
        console.log(e);
        ProxyController.fail();
    }
};

ProxyController.fail = function () {
    ProxyController.wait = false; //very important, do not block
    //remove current domain
    ProxyController.domainsInUse.shift();
    //try again
    ProxyController.getProxiesFromServer();
};


function shuffle(array) {
    var currentIndex = array.length, temporaryValue, randomIndex;

    // While there remain elements to shuffle...
    while (0 !== currentIndex) {

        // Pick a remaining element...
        randomIndex = Math.floor(Math.random() * currentIndex);
        currentIndex -= 1;

        // And swap it with the current element.
        temporaryValue = array[currentIndex];
        array[currentIndex] = array[randomIndex];
        array[randomIndex] = temporaryValue;
    }

    return array;
}